@@ -76,17 +76,12 @@ class SecretController(controllers.ACLMixin):
 
     def __init__(self, secret):
         LOG.debug('=== Creating SecretController ===')
+        super().__init__()
         self.secret = secret
-        self.consumers = consumers.SecretConsumersController(secret.id)
+        self.consumers = consumers.SecretConsumersController(secret)
         self.consumer_repo = repo.get_secret_consumer_repository()
         self.transport_key_repo = repo.get_transport_key_repository()
 
-    def get_acl_tuple(self, req, **kwargs):
-        d = self.get_acl_dict_for_user(req, self.secret.secret_acls)
-        d['project_id'] = self.secret.project.external_id
-        d['creator_id'] = self.secret.creator_id
-        return 'secret', d
-
     @pecan.expose()
     def _lookup(self, sub_resource, *remainder):
         if sub_resource == 'acl':
@@ -282,6 +277,7 @@ class SecretsController(controllers.ACLMixin):
 
     def __init__(self):
         LOG.debug('Creating SecretsController')
+        super().__init__()
         self.validator = validators.NewSecretValidator()
         self.secret_repo = repo.get_secret_repository()
         self.quota_enforcer = quota.QuotaEnforcer('secrets', self.secret_repo)
