@@ -10,15 +10,53 @@
 #  License for the specific language governing permissions and limitations
 #  under the License.
 
+from oslo_log import versionutils
 from oslo_policy import policy
 
+from barbican.common.policies import base
 
-_READER = "role:reader"
+
+deprecated_secretstores_get = policy.DeprecatedRule(
+    name='secretstores:get',
+    check_str='rule:all_users',
+    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
+    deprecated_since=versionutils.deprecated.WALLABY
+)
+deprecated_secretstores_get_global = policy.DeprecatedRule(
+    name='secretstores:get_global_default',
+    check_str='rule:all_users',
+    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
+    deprecated_since=versionutils.deprecated.WALLABY
+)
+deprecated_secretstores_get_preferred = policy.DeprecatedRule(
+    name='secretstores:get_preferred',
+    check_str='rule:all_users',
+    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
+    deprecated_since=versionutils.deprecated.WALLABY
+)
+deprecated_secretstores_preferred_post = policy.DeprecatedRule(
+    name='secretstore_preferred:post',
+    check_str='rule:admin',
+    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
+    deprecated_since=versionutils.deprecated.WALLABY
+)
+deprecated_secretstores_preferred_delete = policy.DeprecatedRule(
+    name='secretstore_preferred:delete',
+    check_str='rule:admin',
+    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
+    deprecated_since=versionutils.deprecated.WALLABY
+)
+deprecated_secretstore_get = policy.DeprecatedRule(
+    name='secretstore:get',
+    check_str='rule:all_users',
+    deprecated_reason=base.LEGACY_POLICY_DEPRECATION,
+    deprecated_since=versionutils.deprecated.WALLABY
+)
 
 rules = [
     policy.DocumentedRuleDefault(
         name='secretstores:get',
-        check_str=f'rule:all_users or {_READER}',
+        check_str='True:%(enforce_new_defaults)s and role:reader',
         scope_types=['project', 'system'],
         description='Get list of available secret store backends.',
         operations=[
@@ -26,11 +64,12 @@ rules = [
                 'path': '/v1/secret-stores',
                 'method': 'GET'
             }
-        ]
+        ],
+        deprecated_rule=deprecated_secretstores_get
     ),
     policy.DocumentedRuleDefault(
         name='secretstores:get_global_default',
-        check_str=f'rule:all_users or {_READER}',
+        check_str='True:%(enforce_new_defaults)s and role:reader',
         scope_types=['project', 'system'],
         description='Get a reference to the secret store that is used as ' +
                     'default secret store backend for the deployment.',
@@ -39,11 +78,12 @@ rules = [
                 'path': '/v1/secret-stores/global-default',
                 'method': 'GET'
             }
-        ]
+        ],
+        deprecated_rule=deprecated_secretstores_get_global
     ),
     policy.DocumentedRuleDefault(
         name='secretstores:get_preferred',
-        check_str=f'rule:all_users or {_READER}',
+        check_str='True:%(enforce_new_defaults)s and role:reader',
         scope_types=['project', 'system'],
         description='Get a reference to the preferred secret store if ' +
                     'assigned previously.',
@@ -52,11 +92,12 @@ rules = [
                 'path': '/v1/secret-stores/preferred',
                 'method': 'GET'
             }
-        ]
+        ],
+        deprecated_rule=deprecated_secretstores_get_preferred
     ),
     policy.DocumentedRuleDefault(
         name='secretstore_preferred:post',
-        check_str='rule:admin',
+        check_str='True:%(enforce_new_defaults)s and role:admin',
         scope_types=['project'],
         description='Set a secret store backend to be preferred store ' +
                     'backend for their project.',
@@ -65,11 +106,12 @@ rules = [
                 'path': '/v1/secret-stores/{ss-id}/preferred',
                 'method': 'POST'
             }
-        ]
+        ],
+        deprecated_rule=deprecated_secretstores_preferred_post
     ),
     policy.DocumentedRuleDefault(
         name='secretstore_preferred:delete',
-        check_str='rule:admin',
+        check_str='True:%(enforce_new_defaults)s and role:admin',
         scope_types=['project'],
         description='Remove preferred secret store backend setting for ' +
                     'their project.',
@@ -78,11 +120,12 @@ rules = [
                 'path': '/v1/secret-stores/{ss-id}/preferred',
                 'method': 'DELETE'
             }
-        ]
+        ],
+        deprecated_rule=deprecated_secretstores_preferred_delete
     ),
     policy.DocumentedRuleDefault(
         name='secretstore:get',
-        check_str=f'rule:all_users or {_READER}',
+        check_str='True:%(enforce_new_defaults)s and role:reader',
         scope_types=['project', 'system'],
         description='Get details of secret store by its ID.',
         operations=[
@@ -90,7 +133,8 @@ rules = [
                 'path': '/v1/secret-stores/{ss-id}',
                 'method': 'GET'
             }
-        ]
+        ],
+        deprecated_rule=deprecated_secretstore_get
     ),
 ]
 
