@@ -729,8 +729,8 @@ class ContainerConsumerValidator(ValidatorBase):
         self.schema = {
             "type": "object",
             "properties": {
-                "URL": {"type": "string", "minLength": 1},
-                "name": {"type": "string", "maxLength": 255, "minLength": 1}
+                "URL": {"type": "string", "maxLength": 255, "minLength": 1},
+                "name": {"type": "string", "maxLength": 36, "minLength": 1}
             },
             "required": ["name", "URL"]
         }
