@@ -36,17 +36,31 @@ p11_crypto_plugin_group = cfg.OptGroup(name='p11_crypto_plugin',
 p11_crypto_plugin_opts = [
     cfg.StrOpt('library_path',
                help=u._('Path to vendor PKCS11 library')),
+    cfg.StrOpt('token_serial_number',
+               help=u._('Token serial number used to identify the token to be '
+                        'used.')),
+    cfg.StrOpt('token_label',
+               deprecated_for_removal=True,
+               help=u._('DEPRECATED: Use token_labels instead. '
+                        'Token label used to identify the token to '
+                        'be used.')),
+    cfg.ListOpt('token_labels',
+                help=u._('List of labels for one or more tokens to be used. '
+                         'Typically this is a single label, but some HSM '
+                         'devices may require more than one label for Load '
+                         'Balancing or High Availability configurations.')),
     cfg.StrOpt('login',
                help=u._('Password to login to PKCS11 session'),
                secret=True),
     cfg.StrOpt('mkek_label',
-               help=u._('Master KEK label (used in the HSM)')),
+               help=u._('Master KEK label (as stored in the HSM)')),
     cfg.IntOpt('mkek_length',
                help=u._('Master KEK length in bytes.')),
     cfg.StrOpt('hmac_label',
-               help=u._('HMAC label (used in the HSM)')),
+               help=u._('Master HMAC Key label (as stored in the HSM)')),
     cfg.IntOpt('slot_id',
-               help=u._('HSM Slot ID'),
+               help=u._('(Optional) HSM Slot ID that contains the token '
+                        'device to be used.'),
                default=1),
     cfg.BoolOpt('rw_session',
                 help=u._('Flag for Read/Write Sessions'),
@@ -60,9 +74,18 @@ p11_crypto_plugin_opts = [
     cfg.IntOpt('pkek_cache_limit',
                help=u._('Project KEK Cache Item Limit'),
                default=100),
-    cfg.StrOpt('algorithm',
-               help=u._('Secret encryption algorithm'),
-               default='VENDOR_SAFENET_CKM_AES_GCM'),
+    cfg.StrOpt('encryption_mechanism',
+               help=u._('Secret encryption mechanism'),
+               default='CKM_AES_CBC', deprecated_name='algorithm'),
+    cfg.StrOpt('hmac_key_type',
+               help=u._('HMAC Key Type'),
+               default='CKK_AES'),
+    cfg.StrOpt('hmac_keygen_mechanism',
+               help=u._('HMAC Key Generation Algorithm'),
+               default='CKM_AES_KEY_GEN'),
+    cfg.StrOpt('hmac_keywrap_mechanism',
+               help=u._('HMAC key wrap mechanism'),
+               default='CKM_SHA256_HMAC'),
     cfg.StrOpt('seed_file',
                help=u._('File to pull entropy for seeding RNG'),
                default=''),
@@ -72,8 +95,16 @@ p11_crypto_plugin_opts = [
     cfg.StrOpt('plugin_name',
                help=u._('User friendly plugin name'),
                default='PKCS11 HSM'),
-    cfg.BoolOpt('generate_iv',
-                help=u._('Flag for plugin generated iv case'),
+    cfg.BoolOpt('aes_gcm_generate_iv',
+                help=u._('Generate IVs for CKM_AES_GCM mechanism.'),
+                default=True, deprecated_name='generate_iv'),
+    cfg.BoolOpt('always_set_cka_sensitive',
+                help=u._('Always set CKA_SENSITIVE=CK_TRUE including '
+                         'CKA_EXTRACTABLE=CK_TRUE keys.'),
+                default=True),
+    cfg.BoolOpt('os_locking_ok',
+                help=u._('Enable CKF_OS_LOCKING_OK flag when initializing the '
+                         'PKCS#11 client library.'),
                 default=False),
 ]
 CONF.register_group(p11_crypto_plugin_group)
@@ -85,6 +116,11 @@ def list_opts():
     yield p11_crypto_plugin_group, p11_crypto_plugin_opts
 
 
+def register_opts(conf):
+    for group, options in list_opts():
+        conf.register_opts(options, group)
+
+
 def json_dumps_compact(data):
     return json.dumps(data, separators=(',', ':'))
 
@@ -97,20 +133,41 @@ class P11CryptoPlugin(plugin.CryptoPluginBase):
     def __init__(self, conf=CONF, ffi=None, pkcs11=None):
         self.conf = conf
         plugin_conf = conf.p11_crypto_plugin
-        if plugin_conf.library_path is None:
-            raise ValueError(u._("library_path is required"))
-
-        # Use specified or create new pkcs11 object
-        self.pkcs11 = pkcs11 or self._create_pkcs11(plugin_conf, ffi)
 
         # Save conf arguments
+        if plugin_conf.library_path is None:
+            raise ValueError(u._("library_path is required"))
+        self.library_path = plugin_conf.library_path
+
+        self.encryption_mechanism = plugin_conf.encryption_mechanism
+        self.generate_iv = plugin_conf.aes_gcm_generate_iv
+        self.cka_sensitive = plugin_conf.always_set_cka_sensitive
+        self.mkek_key_type = 'CKK_AES'
         self.mkek_length = plugin_conf.mkek_length
         self.mkek_label = plugin_conf.mkek_label
         self.hmac_label = plugin_conf.hmac_label
+        self.hmac_key_type = plugin_conf.hmac_key_type
+        self.hmac_keygen_mechanism = plugin_conf.hmac_keygen_mechanism
+        self.hmac_keywrap_mechanism = plugin_conf.hmac_keywrap_mechanism
+        self.os_locking_ok = plugin_conf.os_locking_ok
         self.pkek_length = plugin_conf.pkek_length
         self.pkek_cache_ttl = plugin_conf.pkek_cache_ttl
         self.pkek_cache_limit = plugin_conf.pkek_cache_limit
-        self.algorithm = plugin_conf.algorithm
+        self.rw_session = plugin_conf.rw_session
+        self.seed_file = plugin_conf.seed_file
+        self.seed_length = plugin_conf.seed_length
+        self.slot_id = plugin_conf.slot_id
+        self.login = plugin_conf.login
+        self.token_serial_number = plugin_conf.token_serial_number
+        self.token_labels = plugin_conf.token_labels or list()
+        if plugin_conf.token_label:
+            LOG.warning('Using deprecated option "token_label". Please update '
+                        'your configuration file.')
+            if plugin_conf.token_label not in self.token_labels:
+                self.token_labels.append(plugin_conf.token_label)
+
+        # Use specified or create new pkcs11 object
+        self.pkcs11 = pkcs11 or self._create_pkcs11(ffi)
 
         self._configure_object_cache()
 
@@ -150,6 +207,8 @@ class P11CryptoPlugin(plugin.CryptoPluginBase):
         # Wrap pkcs11 calls to enable a single retry when exceptions are raised
         # that can be fixed by reinitializing the pkcs11 library
         try:
+            if self.pkcs11 is None:
+                self._reinitialize_pkcs11()
             return func(*args, **kwargs)
         except (exception.PKCS11Exception) as pe:
             LOG.warning("Reinitializing PKCS#11 library: %s", pe)
@@ -167,9 +226,10 @@ class P11CryptoPlugin(plugin.CryptoPluginBase):
             if 'session' in locals():
                 self._return_session(session)
 
-        kek_meta_extended = json_dumps_compact(
-            {'iv': base64.b64encode(ct_data['iv'])}
-        )
+        kek_meta_extended = json_dumps_compact({
+            'iv': base64.b64encode(ct_data['iv']),
+            'mechanism': self.encryption_mechanism
+        })
         return plugin.ResponseDTO(ct_data['ct'], kek_meta_extended)
 
     def _decrypt(self, decrypt_dto, kek_meta_dto, kek_meta_extended,
@@ -177,11 +237,12 @@ class P11CryptoPlugin(plugin.CryptoPluginBase):
         kek = self._load_kek_from_meta_dto(kek_meta_dto)
         meta_extended = json.loads(kek_meta_extended)
         iv = base64.b64decode(meta_extended['iv'])
+        mech = meta_extended['mechanism']
 
         try:
             session = self._get_session()
             pt_data = self.pkcs11.decrypt(
-                kek, iv, decrypt_dto.encrypted, session
+                mech, kek, iv, decrypt_dto.encrypted, session
             )
         finally:
             if 'session' in locals():
@@ -216,7 +277,8 @@ class P11CryptoPlugin(plugin.CryptoPluginBase):
                 self._return_session(session)
 
         kek_meta_extended = json_dumps_compact(
-            {'iv': base64.b64encode(ct_data['iv'])}
+            {'iv': base64.b64encode(ct_data['iv']),
+             'mechanism': self.encryption_mechanism}
         )
         return plugin.ResponseDTO(ct_data['ct'], kek_meta_extended)
 
@@ -234,8 +296,8 @@ class P11CryptoPlugin(plugin.CryptoPluginBase):
         self.caching_session_lock = threading.RLock()
 
         # Cache master keys
-        self._get_master_key(self.mkek_label)
-        self._get_master_key(self.hmac_label)
+        self._get_master_key(self.mkek_key_type, self.mkek_label)
+        self._get_master_key(self.hmac_key_type, self.hmac_label)
 
     def _pkek_cache_add(self, kek, label):
         with self.pkek_cache_lock:
@@ -276,24 +338,30 @@ class P11CryptoPlugin(plugin.CryptoPluginBase):
             else:
                 break
 
-    def _create_pkcs11(self, plugin_conf, ffi=None):
+    def _create_pkcs11(self, ffi=None):
         seed_random_buffer = None
-        if plugin_conf.seed_file:
-            with open(plugin_conf.seed_file, 'rb') as f:
-                seed_random_buffer = f.read(plugin_conf.seed_length)
+        if self.seed_file:
+            with open(self.seed_file, 'rb') as f:
+                seed_random_buffer = f.read(self.seed_length)
         return pkcs11.PKCS11(
-            library_path=plugin_conf.library_path,
-            login_passphrase=plugin_conf.login,
-            rw_session=plugin_conf.rw_session,
-            slot_id=plugin_conf.slot_id,
+            library_path=self.library_path,
+            login_passphrase=self.login,
+            rw_session=self.rw_session,
+            slot_id=self.slot_id,
+            encryption_mechanism=self.encryption_mechanism,
             ffi=ffi,
-            algorithm=plugin_conf.algorithm,
             seed_random_buffer=seed_random_buffer,
-            generate_iv=plugin_conf.generate_iv,
+            generate_iv=self.generate_iv,
+            always_set_cka_sensitive=self.cka_sensitive,
+            hmac_keywrap_mechanism=self.hmac_keywrap_mechanism,
+            token_serial_number=self.token_serial_number,
+            token_labels=self.token_labels,
+            os_locking_ok=self.os_locking_ok
         )
 
     def _reinitialize_pkcs11(self):
-        self.pkcs11.finalize()
+        if self.pkcs11 is not None:
+            self.pkcs11.finalize()
         self.pkcs11 = None
 
         with self.caching_session_lock:
@@ -305,7 +373,7 @@ class P11CryptoPlugin(plugin.CryptoPluginBase):
         with self.mk_cache_lock:
             self.mk_cache.clear()
 
-        self.pkcs11 = self._create_pkcs11(self.conf.p11_crypto_plugin)
+        self.pkcs11 = self._create_pkcs11()
         self._configure_object_cache()
 
     def _get_session(self):
@@ -314,13 +382,13 @@ class P11CryptoPlugin(plugin.CryptoPluginBase):
     def _return_session(self, session):
         self.pkcs11.return_session(session)
 
-    def _get_master_key(self, label):
+    def _get_master_key(self, key_type, label):
         with self.mk_cache_lock:
             session = self.caching_session
             key = self.mk_cache.get(label, None)
             if key is None:
                 with self.caching_session_lock:
-                    key = self.pkcs11.get_key_handle(label, session)
+                    key = self.pkcs11.get_key_handle(key_type, label, session)
                 if key is None:
                     raise exception.P11CryptoKeyHandleException(
                         u._("Could not find key labeled {0}").format(label)
@@ -350,8 +418,8 @@ class P11CryptoPlugin(plugin.CryptoPluginBase):
                 with self.caching_session_lock:
                     session = self.caching_session
                     # Get master keys
-                    mkek = self._get_master_key(mkek_label)
-                    mkhk = self._get_master_key(hmac_label)
+                    mkek = self._get_master_key(self.mkek_key_type, mkek_label)
+                    mkhk = self._get_master_key(self.hmac_key_type, hmac_label)
 
                     # Verify HMAC
                     self.pkcs11.verify_hmac(mkhk, hmac, kek_data, session)
@@ -368,11 +436,13 @@ class P11CryptoPlugin(plugin.CryptoPluginBase):
         with self.caching_session_lock:
             session = self.caching_session
             # Get master keys
-            mkek = self._get_master_key(self.mkek_label)
-            mkhk = self._get_master_key(self.hmac_label)
+            mkek = self._get_master_key(self.mkek_key_type, self.mkek_label)
+            mkhk = self._get_master_key(self.hmac_key_type, self.hmac_label)
 
             # Generate KEK
-            kek = self.pkcs11.generate_key(key_length, session, encrypt=True)
+            kek = self.pkcs11.generate_key(
+                'CKK_AES', key_length, 'CKM_AES_KEY_GEN', session, encrypt=True
+            )
 
             # Wrap KEK
             wkek = self.pkcs11.wrap_key(mkek, kek, session)
@@ -391,32 +461,3 @@ class P11CryptoPlugin(plugin.CryptoPluginBase):
             'mkek_label': self.mkek_label,
             'hmac_label': self.hmac_label
         }
-
-    def _generate_mkek(self, key_length, key_label):
-        with self.mk_cache_lock, self.caching_session_lock:
-            session = self.caching_session
-            if key_label in self.mk_cache or \
-                    self.pkcs11.get_key_handle(key_label, session) is not None:
-                raise exception.P11CryptoPluginKeyException(
-                    u._("A master key with that label already exists")
-                )
-            mk = self.pkcs11.generate_key(
-                key_length, session, key_label,
-                encrypt=True, wrap=True, master_key=True
-            )
-            self.mk_cache[key_label] = mk
-        return mk
-
-    def _generate_mkhk(self, key_length, key_label):
-        with self.mk_cache_lock, self.caching_session_lock:
-            session = self.caching_session
-            if key_label in self.mk_cache or \
-                    self.pkcs11.get_key_handle(key_label, session) is not None:
-                raise exception.P11CryptoPluginKeyException(
-                    u._("A master key with that label already exists")
-                )
-            mk = self.pkcs11.generate_key(
-                key_length, session, key_label, sign=True, master_key=True
-            )
-            self.mk_cache[key_label] = mk
-        return mk
