@@ -189,8 +189,7 @@ class SimpleCryptoPlugin(c.CryptoPluginBase):
                  mode=None):
         if type_enum == c.PluginSupportTypes.ENCRYPT_DECRYPT:
             return True
-
-        if type_enum == c.PluginSupportTypes.SYMMETRIC_KEY_GENERATION:
+        elif type_enum == c.PluginSupportTypes.SYMMETRIC_KEY_GENERATION:
             return self._is_algorithm_supported(algorithm,
                                                 bit_length,
                                                 mode)
@@ -234,7 +233,7 @@ class SimpleCryptoPlugin(c.CryptoPluginBase):
             length_factor = 2
 
         if (algorithm.lower() in c.PluginSupportTypes.SYMMETRIC_ALGORITHMS
-            and bit_length/length_factor
+            and bit_length / length_factor
                 in c.PluginSupportTypes.SYMMETRIC_KEY_LENGTHS):
             return True
         elif (algorithm.lower() in c.PluginSupportTypes.ASYMMETRIC_ALGORITHMS
