@@ -15,6 +15,7 @@
 
 import base64
 import datetime
+import fnmatch
 import os
 import re
 import subprocess  # nosec
@@ -23,7 +24,6 @@ import uuid
 
 from OpenSSL import crypto
 from oslo_config import cfg
-from oslo_utils import fnmatch
 from oslo_utils import uuidutils
 
 from barbican.common import config
