@@ -12,8 +12,9 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from unittest import mock
+
 import fixtures
-import mock
 
 from barbican.cmd import barbican_manage as manager
 from barbican.tests import utils
