@@ -15,6 +15,7 @@ limitations under the License.
 """
 import logging
 import os
+import six
 
 
 class BaseBehaviors(object):
@@ -31,7 +32,8 @@ class BaseBehaviors(object):
             json_data = response.json()
         except ValueError as e:
             self.LOG.exception(e)
-            self.LOG.error("Error converting response to JSON: %s", e.message)
+            self.LOG.error("Error converting response to JSON: %s",
+                           six.text_type(e))
             self.LOG.error("Response Content: %s", response.content)
 
         return json_data
